﻿/*
 Copyright (C) 2016 Gottfried Grosshans

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.
 */

#include <avr/sleep.h>
#include <avr/wdt.h>
#include <SPI.h>
#include "RF24.h"
#include <util/delay.h>

//
// Change the value of *channel* for another room!
//

byte const    channel = 55;

// Constants, Variables

byte const    ventopen = 6;
byte const    ventclose = 5;
byte const    led = 7;
unsigned long angle = 999;

//
// Hardware configuration
//

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10

RF24TINY radio(9,10);

//
// Topology
//

// Radio pipe addresses for the 2 nodes to communicate.
const uint64_t pipes[2] = { 0xF0F0F0F0E1LL, 0xF0F0F0F0D2LL };

// watchdog interrupt
ISR (WDT_vect)
{
   wdt_disable();  // disable watchdog
}  // end of WDT_vect

void setup(void)
{
  radio.begin();

  radio.setDataRate(RF24_250KBPS);
  radio.setChannel(channel);

  // optionally, increase the delay between retries & # of retries
  radio.setRetries(15,15);

  // optionally, reduce the payload size.  seems to
  // improve reliability
  radio.setPayloadSize(8);

  //
  // Open pipes to other nodes for communication
  //

  // This simple sketch opens two pipes for these two nodes to communicate
  // back and forth.
  // Open 'our' pipe for writing
  // Open the 'other' pipe for reading,
  // in position #1 (we can have up to 5 pipes open for reading)


    radio.openWritingPipe(pipes[1]);
    radio.openReadingPipe(1,pipes[0]);
  //
  // Start listening
  //

}

// LED blinker

void blinkled () {
  pinMode(led, OUTPUT);
  digitalWrite(led, HIGH);
  _delay_ms(500);
  pinMode(led, INPUT);
  _delay_ms(500);
}

// Another LED blinker

void blinkledshort () {
  pinMode(led, OUTPUT);
  digitalWrite(led, HIGH);
  _delay_ms(250);
  pinMode(led, INPUT);
  _delay_ms(250);
}

// Close valve

void vclose () {
  pinMode(ventopen, OUTPUT);
  pinMode(ventclose, OUTPUT);
  digitalWrite(ventopen, LOW);
  digitalWrite(ventclose, HIGH);
//  analogWrite(ventclose, 150);
  _delay_ms(500);
  pinMode(ventclose, INPUT);
  pinMode(ventopen, INPUT);
}

// Open valve

void vopen () {
  pinMode(ventclose, OUTPUT);
  pinMode(ventopen, OUTPUT);
  digitalWrite(ventclose, LOW);
  digitalWrite(ventopen, HIGH);
//  analogWrite(ventopen, 150);
  _delay_ms(1000);
  pinMode(ventopen, INPUT);
  pinMode(ventclose, INPUT);
}

// Deal with transceiver

void dealradio(void)
{
    radio.powerUp ();
    radio.startListening();
    boolean hit = false;

    for (int x = 10; x > 0; x--) {
      _delay_ms(5);
    // if there is data ready
    if ( radio.available() )
    {
      // Dump the payloads until we've gotten everything
      unsigned long got_number;
      bool done = false;
      while (!done)
      {
        // Fetch the payload, and see if this was the last one.
        done = radio.read( &got_number, sizeof(unsigned long) );
        angle = got_number;
	// Delay just a little bit to let the other unit
	// make the transition to receiver
	_delay_ms(20);
      }

      // First, stop listening so we can talk
      radio.stopListening();

      // Send the final one back.
      radio.write( &got_number, sizeof(unsigned long) );

      // Now, resume listening so we catch the next packets.
      radio.startListening();
    }
    }

// Stop radio
    radio.stopListening();
    radio.powerDown ();

// Handle valve with data received
    if (angle == 0) {
      vclose ();
      blinkled ();
    }
    if (angle == 255) {
      vopen ();
      blinkled ();
      blinkled ();
    }
    angle=999;
}

void loop ()
{

  dealradio ();

  // disable ADC
  ADCSRA = 0;

  // clear various "reset" flags
  MCUSR = 0;
  // allow changes, disable reset
  WDTCSR = bit (WDCE) | bit (WDE);
  // set interrupt mode and an interval
  WDTCSR = bit (WDIE) | bit (WDP3) | bit (WDP0); // set WDIE, and 8 seconds delay
  wdt_reset();  // pat the dog

  set_sleep_mode (SLEEP_MODE_PWR_DOWN);
  sleep_enable();

  // turn off brown-out enable in software
  MCUCR = bit (BODS) | bit (BODSE);
  MCUCR = bit (BODS);
  sleep_cpu ();

  // cancel sleep as a precaution
  sleep_disable();

  } // end of loop
